package edu.kit.iti.formal.lights;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Locale;

/**
 * @author Alexander Weigl
 * @version 1 (04.11.19)
 */
public class SVGWriter {
    private static final double FIELD_SIZE = 25f;
    private static final double RADIUS = FIELD_SIZE / 2.f;
    private final Lights lights;
    private final SolutionChecker solutionChecker;
    private PrintStream out;

    SVGWriter(Lights lights, SolutionChecker checker) {
        this.lights = lights;
        this.solutionChecker = checker;
    }

    public static void writeFile(@NotNull String filename,
                                 @NotNull Lights lights,
                                 @Nullable SolutionChecker checker) {
        Locale.setDefault(Locale.ENGLISH);
        var writer = new SVGWriter(lights, checker);
        try (var fw = new FileOutputStream(filename)) {
            writer.out = new PrintStream(fw);
            writer.write();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void write() {
        out.println("<?xml version=\"1.0\" standalone=\"no\"?>");
        out.format("<svg width=\"%f\" height=\"%f\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">",
                lights.getDimension() * FIELD_SIZE,
                lights.getDimension() * FIELD_SIZE
        );
        for (int column = 0; column < lights.getDimension(); column++) {
            for (int row = 0; row < lights.getDimension(); row++) {

                String colour = lights.isBlock(row, column) ? "black" :
                        solutionChecker.isValidAndLit(row, column) ? "yellow" : "grey";
                boolean containsLamp = solutionChecker.isValidAndLamp(row, column);

                out.format("<svg y=\"%f\" x=\"%f\" width=\"%f\" height=\"%f\">\n" +
                                "<rect y=\"0\" x=\"0\" width=\"100%%\" height=\"100%%\" stroke=\"lightgrey\" fill=\"%s\" " +
                                "stroke-width=\"2\"/>\n",
                        row * FIELD_SIZE, column * FIELD_SIZE, FIELD_SIZE, FIELD_SIZE, colour);

                if (containsLamp) {
                    out.format("<circle cy=\"50%%\" cx=\"50%%\" r=\"40%%\" fill=\"white\" stroke=\"black\"/>\n");
                }
                if (lights.isConstrainedBlock(row, column)) {
                    var num = lights.getBlockConstraint(row, column);
                    out.format("<text x=\"50%%\" y=\"50%%\" text-anchor=\"middle\"  stroke-width=\"1px\" dy=\".3em\" " +
                            "fill=\"white\">%s</text>\n", num);
                }

                out.format("</svg>");

            }
        }
        out.println("</svg>");
    }
}
